@@ -273,6 +273,7 @@ class NavigationToolbar(NavigationToolbar2QT):
         self.setIconSize(QtCore.QSize(24, 24))
         self.layout().setSpacing(12)
         self.canvas = canvas
+        self.no_push_history = False
 
     def _icon(self, name, *args):
         """
@@ -300,6 +301,34 @@ class NavigationToolbar(NavigationToolbar2QT):
             elif self.mode == _Mode.DELETE_WP:
                 self.canvas.waypoints_interactor.button_release_delete_callback(event)
 
+    def clear_history(self):
+        self._nav_stack.clear()
+        self.push_current()
+        self.set_history_buttons()
+
+    def push_current(self):
+        """Push the current view limits and position onto the stack."""
+        if self.sideview:
+            super(NavigationToolbar, self).push_current()
+        elif self.no_push_history:
+            pass
+        else:
+            self._nav_stack.push(self.canvas.map.kwargs.copy())
+            self.set_history_buttons()
+
+    def _update_view(self):
+        """
+        Update the viewlim and position from the view and position stack for
+        each axes.
+        """
+        if self.sideview:
+            super(NavigationToolbar, self)._update_view()
+        else:
+            nav_info = self._nav_stack()
+            if nav_info is None:
+                return
+            self.canvas.redraw_map(nav_info)
+
     def insert_wp(self, *args):
         """
         activate insert_wp tool
@@ -346,12 +375,18 @@ class NavigationToolbar(NavigationToolbar2QT):
         self._update_buttons_checked()
 
     def release_zoom(self, event):
+        self.no_push_history = True
         super(NavigationToolbar, self).release_zoom(event)
+        self.no_push_history = False
         self.canvas.redraw_map()
+        self.push_current()
 
     def release_pan(self, event):
+        self.no_push_history = True
         super(NavigationToolbar, self).release_pan(event)
+        self.no_push_history = False
         self.canvas.redraw_map()
+        self.push_current()
 
     def mouse_move(self, event):
         """
@@ -359,7 +394,14 @@ class NavigationToolbar(NavigationToolbar2QT):
         """
         if self.mode == _Mode.MOVE_WP:
             self.canvas.waypoints_interactor.motion_notify_callback(event)
-        if not self.sideview:
+
+        if isinstance(self.canvas.waypoints_interactor, mpl_pi.LPathInteractor):
+            if not event.ydata or not event.xdata:
+                self.set_message(self.mode)
+            else:
+                (lat, lon, alt), _ = self.canvas.waypoints_interactor.get_lat_lon(event)
+                self.set_message(f"lat={lat: <6.2f} lon={lon: <7.2f} altitude={alt/100: <.2f}hPa")
+        elif not self.sideview:
             self._update_cursor(event)
 
             if event.inaxes and event.inaxes.get_navigate():
@@ -452,11 +494,14 @@ class MplSideViewCanvas(MplCanvas):
         # Default settings.
         self.settings_dict = {"vertical_extent": (1050, 180),
                               "vertical_axis": "pressure",
+                              "secondary_axis": "no secondary axis",
                               "flightlevels": [],
                               "draw_flightlevels": True,
                               "draw_flighttrack": True,
                               "fill_flighttrack": True,
                               "label_flighttrack": True,
+                              "draw_verticals": True,
+                              "draw_marker": True,
                               "draw_ceiling": True,
                               "colour_ft_vertices": (0, 0, 1, 1),
                               "colour_ft_waypoints": (1, 0, 0, 1),
@@ -466,15 +511,20 @@ class MplSideViewCanvas(MplCanvas):
             self.settings_dict.update(settings)
 
         # Setup the plot.
-        self.p_bot = self.settings_dict["vertical_extent"][0] * 100
-        self.p_top = self.settings_dict["vertical_extent"][1] * 100
+        self.update_vertical_extent_from_settings(init=True)
+
         self.numlabels = numlabels
+        self.ax2 = self.ax.twinx()
+        self.ax.patch.set_facecolor("None")
+        self.ax2.patch.set_facecolor("None")
+        # Main axes instance of mplwidget has zorder 99.
+        self.imgax = self.fig.add_axes(
+            self.ax.get_position(), frameon=True, xticks=[], yticks=[], label="imgax", zorder=0)
         self.setup_side_view()
         # Draw a number of flight level lines.
         self.flightlevels = []
         self.fl_label_list = []
         self.draw_flight_levels()
-        self.imgax = None
         self.image = None
         self.ceiling_alt = []
         # If a waypoints model has been passed, create an interactor on it.
@@ -503,12 +553,13 @@ class MplSideViewCanvas(MplCanvas):
                 redraw_xaxis=self.redraw_xaxis, clear_figure=self.clear_figure
             )
 
-    def redraw_yaxis(self):
-        """ Redraws the y-axis on map after setting the values from sideview options dialog box"""
-
-        self.checknconvert()
-        vaxis = self.settings_dict["vertical_axis"]
-        if vaxis == "pressure":
+    def _determine_ticks_labels(self, typ):
+        if typ == "no secondary axis":
+            major_ticks = []
+            minor_ticks = []
+            labels = []
+            ylabel = ""
+        elif typ == "pressure":
             # Compute the position of major and minor ticks. Major ticks are labelled.
             major_ticks = self._pres_maj[(self._pres_maj <= self.p_bot) & (self._pres_maj >= self.p_top)]
             minor_ticks = self._pres_min[(self._pres_min <= self.p_bot) & (self._pres_min >= self.p_top)]
@@ -518,8 +569,8 @@ class MplSideViewCanvas(MplCanvas):
                 labels = ["" if x.split(".")[-1][0] in "975" else x for x in labels]
             elif len(labels) > 10:
                 labels = ["" if x.split(".")[-1][0] in "9" else x for x in labels]
-            self.ax.set_ylabel("pressure (hPa)")
-        elif vaxis == "pressure altitude":
+            ylabel = "pressure (hPa)"
+        elif typ == "pressure altitude":
             bot_km = thermolib.pressure2flightlevel(self.p_bot) * 0.03048
             top_km = thermolib.pressure2flightlevel(self.p_top) * 0.03048
             ma_dist, mi_dist = 4, 1.0
@@ -532,8 +583,8 @@ class MplSideViewCanvas(MplCanvas):
             major_ticks = thermolib.flightlevel2pressure_a(major_heights / 0.03048)
             minor_ticks = thermolib.flightlevel2pressure_a(minor_heights / 0.03048)
             labels = major_heights
-            self.ax.set_ylabel("pressure altitude (km)")
-        elif vaxis == "flight level":
+            ylabel = "pressure altitude (km)"
+        elif typ == "flight level":
             bot_km = thermolib.pressure2flightlevel(self.p_bot) * 0.03048
             top_km = thermolib.pressure2flightlevel(self.p_top) * 0.03048
             ma_dist, mi_dist = 50, 10
@@ -546,16 +597,33 @@ class MplSideViewCanvas(MplCanvas):
             major_ticks = thermolib.flightlevel2pressure_a(major_fl)
             minor_ticks = thermolib.flightlevel2pressure_a(minor_fl)
             labels = major_fl
-            self.ax.set_ylabel("flight level (hft)")
+            ylabel = "flight level (hft)"
         else:
-            raise RuntimeError(f"Unsupported vertical axis type: '{vaxis}'")
+            raise RuntimeError(f"Unsupported vertical axis type: '{typ}'")
+        return ylabel, major_ticks, minor_ticks, labels
 
-        # Draw ticks and tick labels.
-        self.ax.set_yticks(minor_ticks, minor=True)
-        self.ax.set_yticks(major_ticks, minor=False)
-        self.ax.set_yticklabels([], minor=True, fontsize=10)
-        self.ax.set_yticklabels(labels, minor=False, fontsize=10)
-        self.ax.set_ylim(self.p_bot, self.p_top)
+    def redraw_yaxis(self):
+        """ Redraws the y-axis on map after setting the values from sideview options dialog box"""
+
+        vaxis = self.settings_dict["vertical_axis"]
+        vaxis2 = self.settings_dict["secondary_axis"]
+
+        for ax, typ in zip((self.ax, self.ax2), (vaxis, vaxis2)):
+            ylabel, major_ticks, minor_ticks, labels = self._determine_ticks_labels(typ)
+
+            ax.set_ylabel(ylabel)
+            ax.set_yticks(minor_ticks, minor=True)
+            ax.set_yticks(major_ticks, minor=False)
+            ax.set_yticklabels([], minor=True, fontsize=10)
+            ax.set_yticklabels(labels, minor=False, fontsize=10)
+            ax.set_ylim(self.p_bot, self.p_top)
+
+        if vaxis2 == "no secondary axis":
+            self.fig.subplots_adjust(left=0.08, right=0.96, top=0.9, bottom=0.14)
+            self.imgax.set_position(self.ax.get_position())
+        else:
+            self.fig.subplots_adjust(left=0.08, right=0.92, top=0.9, bottom=0.14)
+            self.imgax.set_position(self.ax.get_position())
 
     def setup_side_view(self):
         """Set up a vertical section view.
@@ -563,19 +631,15 @@ class MplSideViewCanvas(MplCanvas):
         Vertical cross section code (log-p axis etc.) taken from
         mss_batch_production/visualisation/mpl_vsec.py.
         """
-        self.checknconvert()
 
-        ax = self.ax
-        self.fig.subplots_adjust(left=0.08, right=0.96, top=0.9, bottom=0.14)
+        self.ax.set_title("vertical flight profile", horizontalalignment="left", x=0)
+        self.ax.grid(b=True)
+        self.ax.set_xlabel("lat/lon")
 
-        ax.set_title("vertical flight profile", horizontalalignment="left", x=0)
-        ax.set_yscale("log")
+        for ax in (self.ax, self.ax2):
+            ax.set_yscale("log")
+            ax.set_ylim(self.p_bot, self.p_top)
 
-        # Set axis limits and draw grid for major ticks.
-        ax.set_ylim(self.p_bot, self.p_top)
-        ax.grid(b=True)
-
-        ax.set_xlabel("lat/lon")
         self.redraw_yaxis()
 
     def clear_figure(self):
@@ -620,51 +684,46 @@ class MplSideViewCanvas(MplCanvas):
             vertices = self.waypoints_interactor.pathpatch.get_path().vertices
             vx, vy = list(zip(*vertices))
             wpd = self.waypoints_model.all_waypoint_data()
-            xs, ys = [], []
-            aircraft = self.waypoints_model.performance_settings["aircraft"]
-            for i in range(len(wpd) - 1):
-                weight = np.linspace(wpd[i].weight, wpd[i + 1].weight, 5, endpoint=False)
-                ceil = [aircraft.get_ceiling_altitude(_w) for _w in weight]
-                xs.extend(np.linspace(vx[i], vx[i + 1], 5, endpoint=False))
-                ys.extend(ceil)
-            xs.append(vx[-1])
-            ys.append(aircraft.get_ceiling_altitude(wpd[-1].weight))
+            if len(wpd) > 0:
+                xs, ys = [], []
+                aircraft = self.waypoints_model.performance_settings["aircraft"]
+                for i in range(len(wpd) - 1):
+                    weight = np.linspace(wpd[i].weight, wpd[i + 1].weight, 5, endpoint=False)
+                    ceil = [aircraft.get_ceiling_altitude(_w) for _w in weight]
+                    xs.extend(np.linspace(vx[i], vx[i + 1], 5, endpoint=False))
+                    ys.extend(ceil)
+                xs.append(vx[-1])
+                ys.append(aircraft.get_ceiling_altitude(wpd[-1].weight))
 
-            self.ceiling_alt = self.ax.plot(
-                xs, thermolib.flightlevel2pressure_a(np.asarray(ys)),
-                color="k", ls="--")
-            self.update_ceiling(
-                self.settings_dict["draw_ceiling"] and self.waypoints_model.performance_settings["visible"],
-                self.settings_dict["colour_ceiling"])
+                self.ceiling_alt = self.ax.plot(
+                    xs, thermolib.flightlevel2pressure_a(np.asarray(ys)),
+                    color="k", ls="--")
+                self.update_ceiling(
+                    self.settings_dict["draw_ceiling"] and self.waypoints_model.performance_settings["visible"],
+                    self.settings_dict["colour_ceiling"])
+
+            # Remove all vertical lines
+            vertical_lines = [line for line in self.ax.lines if
+                              all(x == line.get_path().vertices[0, 0] for x in line.get_path().vertices[:, 0])]
+            for line in vertical_lines:
+                self.ax.lines.remove(line)
+
+            # Add vertical lines
+            if self.settings_dict["draw_verticals"]:
+                ipoint = 0
+                highlight = [[wp.lat, wp.lon] for wp in self.waypoints_model.waypoints]
+                for i, (lat, lon) in enumerate(zip(lats, lons)):
+                    if (ipoint < len(highlight) and
+                            np.hypot(lat - highlight[ipoint][0],
+                                     lon - highlight[ipoint][1]) < 2E-10):
+                        self.ax.axvline(i, color='k', linewidth=2, linestyle='--', alpha=0.5)
+                        ipoint += 1
 
         self.draw()
 
-    def set_vertical_extent(self, pbot, ptop):
-        """Set the vertical extent of the view to the specified pressure
-           values (hPa) and redraw the plot.
-        """
-        self.checknconvert()
-        changed = False
-        if self.p_bot != pbot * 100:
-            self.p_bot = pbot * 100
-            changed = True
-        if self.p_top != ptop * 100:
-            self.p_top = ptop * 100
-            changed = True
-        if changed:
-            if self.image is not None:
-                self.image.remove()
-                self.image = None
-            self.setup_side_view()
-            self.waypoints_interactor.redraw_figure()
-        else:
-            self.redraw_yaxis()
-
     def get_vertical_extent(self):
         """Returns the bottom and top pressure (hPa) of the plot.
         """
-        self.checknconvert()
-
         return (self.p_bot // 100), (self.p_top // 100)
 
     def draw_flight_levels(self):
@@ -717,19 +776,21 @@ class MplSideViewCanvas(MplCanvas):
     def set_settings(self, settings):
         """Apply settings to view.
         """
+        vertical_lines = self.settings_dict["draw_verticals"]
         if settings is not None:
             self.settings_dict.update(settings)
         settings = self.settings_dict
         self.set_flight_levels(settings["flightlevels"])
-        self.set_vertical_extent(*settings["vertical_extent"])
         self.set_flight_levels_visible(settings["draw_flightlevels"])
         self.update_ceiling(
             settings["draw_ceiling"] and (
                 self.waypoints_model is not None and
                 self.waypoints_model.performance_settings["visible"]),
             settings["colour_ceiling"])
+        self.update_vertical_extent_from_settings()
 
         if self.waypoints_interactor is not None:
+            self.waypoints_interactor.line.set_marker("o" if settings["draw_marker"] else None)
             self.waypoints_interactor.set_vertices_visible(
                 settings["draw_flighttrack"])
             self.waypoints_interactor.set_path_color(
@@ -741,6 +802,11 @@ class MplSideViewCanvas(MplCanvas):
             self.waypoints_interactor.set_labels_visible(
                 settings["label_flighttrack"])
 
+        if self.waypoints_model is not None and self.waypoints_interactor is not None \
+                and settings["draw_verticals"] != vertical_lines:
+            self.redraw_xaxis(self.waypoints_interactor.path.ilats, self.waypoints_interactor.path.ilons,
+                              self.waypoints_interactor.path.itimes)
+
         self.settings_dict = settings
 
     def getBBOX(self):
@@ -780,19 +846,6 @@ class MplSideViewCanvas(MplCanvas):
         logging.debug("plotting vertical section image..")
         ix, iy = img.size
         logging.debug("  image size is %dx%d px, format is '%s'", ix, iy, img.format)
-        # Test if the image axes exist. If not, create them.
-        if self.imgax is None:
-            # Disable old white figure background so that the new underlying
-            # axes become visible.
-            self.ax.patch.set_facecolor("None")
-            # self.mpl.canvas.ax.patch.set_alpha(0.5)
-
-            # Add new axes to the plot (imshow doesn't work with logarithmic axes).
-            ax_bbox = self.ax.get_position()
-            # Main axes instance of mplwidget has zorder 99.
-            self.imgax = self.fig.add_axes(ax_bbox, frameon=True,
-                                           xticks=[], yticks=[],
-                                           label="ax2", zorder=0)
 
         # If an image is currently displayed, remove it from the plot.
         if self.image is not None:
@@ -806,16 +859,33 @@ class MplSideViewCanvas(MplCanvas):
         self.draw()
         logging.debug("done.")
 
-    def checknconvert(self):
+    def update_vertical_extent_from_settings(self, init=False):
         """ Checks for current units of axis and convert the upper and lower limit
         to pa(pascals) for the internal computation by code """
 
+        if not init:
+            p_bot_old = self.p_bot
+            p_top_old = self.p_top
+
         if self.settings_dict["vertical_axis"] == "pressure altitude":
             self.p_bot = thermolib.flightlevel2pressure(self.settings_dict["vertical_extent"][0] * 32.80)
             self.p_top = thermolib.flightlevel2pressure(self.settings_dict["vertical_extent"][1] * 32.80)
         elif self.settings_dict["vertical_axis"] == "flight level":
             self.p_bot = thermolib.flightlevel2pressure(self.settings_dict["vertical_extent"][0])
             self.p_top = thermolib.flightlevel2pressure(self.settings_dict["vertical_extent"][1])
+        else:
+            self.p_bot = self.settings_dict["vertical_extent"][0] * 100
+            self.p_top = self.settings_dict["vertical_extent"][1] * 100
+
+        if not init:
+            if (p_bot_old != self.p_bot) or (p_top_old != self.p_top):
+                if self.image is not None:
+                    self.image.remove()
+                    self.image = None
+                self.setup_side_view()
+                self.waypoints_interactor.redraw_figure()
+            else:
+                self.redraw_yaxis()
 
 
 class MplSideViewWidget(MplNavBarWidget):
@@ -836,6 +906,164 @@ class MplSideViewWidget(MplNavBarWidget):
                 action.setEnabled(False)
 
 
+class MplLinearViewCanvas(MplCanvas):
+    """Specialised MplCanvas that draws a linear view of a
+       flight track / list of waypoints.
+    """
+
+    def __init__(self, model=None, numlabels=None):
+        """
+        Arguments:
+        model -- WaypointsTableModel defining the linear section.
+        """
+        if numlabels is None:
+            numlabels = config_loader(dataset='num_labels')
+        super(MplLinearViewCanvas, self).__init__()
+
+        # Setup the plot.
+        self.numlabels = numlabels
+        self.setup_linear_view()
+        # If a waypoints model has been passed, create an interactor on it.
+        self.waypoints_interactor = None
+        self.waypoints_model = None
+        self.basename = "linearview"
+        self.draw()
+        if model:
+            self.set_waypoints_model(model)
+
+    def set_waypoints_model(self, model):
+        """Set the WaypointsTableModel defining the linear section.
+        If no model had been set before, create a new interactor object on the model
+        """
+        self.waypoints_model = model
+        pass
+        if self.waypoints_interactor:
+            self.waypoints_interactor.set_waypoints_model(model)
+        else:
+            # Create a path interactor object. The interactor object connects
+            # itself to the change() signals of the flight track data model.
+            self.waypoints_interactor = mpl_pi.LPathInteractor(
+                self.ax, self.waypoints_model,
+                numintpoints=config_loader(dataset="num_interpolation_points"),
+                clear_figure=self.clear_figure,
+                redraw_xaxis=self.redraw_xaxis
+            )
+        self.redraw_xaxis()
+
+    def setup_linear_view(self):
+        """Set up a linear section view.
+        """
+        self.ax.set_title("Linear flight profile", horizontalalignment="left", x=0)
+        self.fig.subplots_adjust(left=0.08, right=0.96, top=0.9, bottom=0.14)
+
+    def clear_figure(self):
+        logging.debug("path of linear view has changed.. removing invalidated plots")
+        self.ax.figure.clf()
+        self.ax = self.fig.add_subplot(111, zorder=99)
+        self.ax.figure.patch.set_visible(False)
+
+    def getBBOX(self):
+        """Get the bounding box of the view.
+        """
+        # Get the number of (great circle) interpolation points and the
+        # number of labels along the x-axis.
+        if self.waypoints_interactor is not None:
+            num_interpolation_points = \
+                self.waypoints_interactor.get_num_interpolation_points()
+
+        # Return a tuple (num_interpolation_points) as BBOX.
+        bbox = (num_interpolation_points,)
+        return bbox
+
+    def draw_legend(self, img):
+        if img is not None:
+            logging.error("Legends not supported in LinearView mode!")
+            raise NotImplementedError
+
+    def redraw_xaxis(self):
+        """Redraw the x-axis of the linear view on path changes.
+        """
+        if self.waypoints_interactor is not None:
+            lats = self.waypoints_interactor.path.ilats
+            lons = self.waypoints_interactor.path.ilons
+            logging.debug("redrawing x-axis")
+
+            # Re-label x-axis.
+            self.ax.set_xlim(0, len(lats) - 1)
+            # Set xticks so that they display lat/lon. Plot "numlabels" labels.
+            lat_inds = np.arange(len(lats))
+            tick_index_step = len(lat_inds) // self.numlabels
+            self.ax.set_xticks(lat_inds[::tick_index_step])
+            self.ax.set_xticklabels([f'{d[0]:2.1f}, {d[1]:2.1f}'
+                                     for d in zip(lats[::tick_index_step],
+                                                  lons[::tick_index_step])],
+                                    rotation=25, fontsize=10, horizontalalignment="right")
+
+            ipoint = 0
+            highlight = [[wp.lat, wp.lon] for wp in self.waypoints_model.waypoints]
+            for i, (lat, lon) in enumerate(zip(lats, lons)):
+                if (ipoint < len(highlight) and
+                        np.hypot(lat - highlight[ipoint][0],
+                                 lon - highlight[ipoint][1]) < 2E-10):
+                    self.ax.axvline(i, color='k', linewidth=2, linestyle='--', alpha=0.5)
+                    ipoint += 1
+            self.draw()
+
+    def draw_image(self, xmls, colors=None, scales=None):
+        self.clear_figure()
+        offset = 40
+        self.ax.patch.set_visible(False)
+
+        for i, xml in enumerate(xmls):
+            data = xml.find("Data")
+            values = [float(value) for value in data.text.split(",")]
+            unit = data.attrib["unit"]
+            numpoints = int(data.attrib["num_waypoints"])
+
+            if colors:
+                color = colors[i] if len(colors) > i else colors[-1]
+            else:
+                color = "#00AAFF"
+
+            if scales:
+                scale = scales[i] if len(scales) > i else scales[-1]
+            else:
+                scale = "linear"
+
+            par = self.ax.twinx() if i > 0 else self.ax
+            par.set_yscale(scale)
+
+            par.plot(range(numpoints), values, color)
+            if i > 0:
+                par.spines["right"].set_position(("outward", (i - 1) * offset))
+            if unit:
+                par.set_ylabel(unit)
+
+            par.yaxis.label.set_color(color.replace("0x", "#"))
+        self.redraw_xaxis()
+        self.fig.tight_layout()
+        self.fig.subplots_adjust(top=0.85, bottom=0.20)
+        self.draw()
+
+
+class MplLinearViewWidget(MplNavBarWidget):
+    """MplNavBarWidget using an MplLinearViewCanvas as the Matplotlib
+       view instance.
+    """
+
+    def __init__(self, parent=None):
+        super(MplLinearViewWidget, self).__init__(
+            sideview=False, parent=parent, canvas=MplLinearViewCanvas())
+        # Disable some elements of the Matplotlib navigation toolbar.
+        # Available actions: Home, Back, Forward, Pan, Zoom, Subplots,
+        #                    Customize, Save, Insert Waypoint, Delete Waypoint
+        actions = self.navbar.actions()
+        for action in actions[:-1]:
+            if action.text() in ["Home", "Back", "Forward", "Pan", "Zoom", "",
+                                 "Subplots", "Customize", "Mv WP", "Del WP", "Ins WP"]:
+                action.setVisible(False)
+
+
 class MplTopViewCanvas(MplCanvas):
     """Specialised MplCanvas that draws a top view (map), together with a
        flight track, trajectories and other items.
@@ -891,11 +1119,15 @@ class MplTopViewCanvas(MplCanvas):
             # Create a path interactor object. The interactor object connects
             # itself to the change() signals of the flight track data model.
             appearance = self.get_map_appearance()
-            self.waypoints_interactor = mpl_pi.HPathInteractor(
-                self.map, self.waypoints_model,
-                linecolor=appearance["colour_ft_vertices"],
-                markerfacecolor=appearance["colour_ft_waypoints"])
-            self.waypoints_interactor.set_vertices_visible(appearance["draw_flighttrack"])
+            try:
+                self.waypoints_interactor = mpl_pi.HPathInteractor(
+                    self.map, self.waypoints_model,
+                    linecolor=appearance["colour_ft_vertices"],
+                    markerfacecolor=appearance["colour_ft_waypoints"],
+                    show_marker=appearance["draw_marker"])
+                self.waypoints_interactor.set_vertices_visible(appearance["draw_flighttrack"])
+            except IOError as err:
+                logging.error("%s" % err)
 
     def redraw_map(self, kwargs_update=None):
         """Redraw map canvas.
@@ -1066,15 +1298,7 @@ class MplTopViewCanvas(MplCanvas):
     def plot_kml(self, kmloverlay):
         """Plots a satellite track on top of the map.
         """
-        if self.kmloverlay:
-            # If track is currently plotted on the map, remove it.
-            self.kmloverlay.remove()
-            if not kmloverlay:
-                self.kmloverlay = None
-                self.draw()
-        if kmloverlay:
-            # Create a new patch.
-            self.kmloverlay = kmloverlay
+        self.kmloverlay = kmloverlay
 
     def set_map_appearance(self, settings_dict):
         """Apply settings from dictionary 'settings_dict' to the view.
@@ -1087,6 +1311,7 @@ class MplTopViewCanvas(MplCanvas):
                     "fill_waterbodies": True,
                     "fill_continents": True,
                     "draw_flighttrack": True,
+                    "draw_marker": True,
                     "label_flighttrack": True,
                     "colour_water": ((153 / 255.), (255 / 255.), (255 / 255.), (255 / 255.)),
                     "colour_land": ((204 / 255.), (153 / 255.), (102 / 255.), (255 / 255.)),
@@ -1107,6 +1332,7 @@ class MplTopViewCanvas(MplCanvas):
                                              bg_color=settings["colour_water"])
             self.waypoints_interactor.set_path_color(line_color=settings["colour_ft_vertices"],
                                                      marker_facecolor=settings["colour_ft_waypoints"])
+            self.waypoints_interactor.show_marker = settings["draw_marker"]
             self.waypoints_interactor.set_vertices_visible(settings["draw_flighttrack"])
             self.waypoints_interactor.set_labels_visible(settings["label_flighttrack"])
 
@@ -1138,11 +1364,3 @@ class MplTopViewWidget(MplNavBarWidget):
         for action in actions:
             if action.text() in ["Subplots", "Customize"]:
                 action.setEnabled(False)
-            elif action.text() in ["Home", "Back", "Forward"]:
-                action.triggered.connect(self.historyEvent)
-
-    def historyEvent(self):
-        """Slot to react to clicks on one of the history buttons in the
-           navigation toolbar. Redraws the image.
-        """
-        self.canvas.redraw_map()
